"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = featureComponents;

var React = _interopRequireWildcard(require("react"));

var _reactIs = require("react-is");

var _lodash = require("lodash");

var _reactRedux = require("react-redux");

var _reselect = require("reselect");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function featureComponents(options) {
  const {
    getFeatures,
    sortFeatures,
    getComponents
  } = (0, _lodash.defaults)({}, options, {
    getFeatures: state => state ? state.features : {},
    sortFeatures: features => Object.values(features)
  });
  const mapStateToProps = (0, _reselect.createStructuredSelector)({
    features: (0, _reselect.createSelector)(getFeatures, sortFeatures)
  });

  function mergeProps({
    features
  }, dispatchProps, _ref) {
    let {
      children
    } = _ref,
        props = _objectWithoutProperties(_ref, ["children"]);

    const renderedComponents = [];
    features.forEach((feature, featureIndex) => {
      let components = getComponents(feature);
      if (components == null) return;
      if (!Array.isArray(components)) components = [components];
      components.forEach((Comp, index) => {
        const key = featureIndex + ':' + index;

        if (React.isValidElement(Comp)) {
          renderedComponents.push(React.cloneElement(Comp, _objectSpread({}, props, {
            key
          })));
        } else if ((0, _reactIs.isValidElementType)(Comp)) {
          const FeatureComponent = Comp;
          renderedComponents.push(React.createElement(FeatureComponent, _extends({}, props, {
            key: key
          })));
        }
      });
    });
    return {
      children: children instanceof Function ? children(renderedComponents) : React.createElement(React.Fragment, null, renderedComponents)
    };
  }

  const FeatureComponents = ({
    children
  }) => children;

  return (0, _reactRedux.connect)(mapStateToProps, null, mergeProps)(FeatureComponents);
}